@@ -18,6 +18,8 @@
    #error configuration problem, ALLEGRO_CFG_ACODEC_MODAUDIO not set
 #endif
 
+// We use the deprecated duh_render in DUMB 2.
+#define DUMB_DECLARE_DEPRECATED
 #include <dumb.h>
 #include <stdio.h>
 
@@ -163,7 +165,7 @@ static size_t modaudio_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
 
    /* the mod files are stereo and 16-bit */
    const int sample_size = 4;
-   size_t written;
+   size_t written = 0;
    size_t i;
 
    DUMB_IT_SIGRENDERER *it_sig = lib.duh_get_it_sigrenderer(df->sig);
@@ -173,19 +175,18 @@ static size_t modaudio_stream_update(ALLEGRO_AUDIO_STREAM *stream, void *data,
          ? lib.dumb_it_callback_terminate : NULL, NULL);
    }
 
-   written = lib.duh_render(df->sig, 16, 0, 1.0, 65536.0 / 44100.0,
-      buf_size / sample_size, data) * sample_size;
+   while (written < buf_size) {
+      written += lib.duh_render(df->sig, 16, 0, 1.0, 65536.0 / 44100.0,
+         (buf_size - written) / sample_size, &(((char *)data)[written])) * sample_size;
+      if (stream->spl.loop == _ALLEGRO_PLAYMODE_STREAM_ONCE) {
+            break;
+      }
+   }
 
    /* Fill the remainder with silence */
    for (i = written; i < buf_size; ++i)
       ((char *)data)[i] = 0;
 
-   /* Check to see if a loop is set */
-   if (df->loop_start != -1 &&
-      df->loop_end < lib.duh_sigrenderer_get_position(df->sig)) {
-         modaudio_stream_seek(stream, df->loop_start / 65536.0);
-   }
-
    return written;
 }
 
@@ -204,7 +205,7 @@ static bool modaudio_stream_rewind(ALLEGRO_AUDIO_STREAM *stream)
 {
    MOD_FILE *const df = stream->extra;
    lib.duh_end_sigrenderer(df->sig);
-   df->sig = lib.duh_start_sigrenderer(df->duh, 0, 2, 0);
+   df->sig = lib.duh_start_sigrenderer(df->duh, 0, 2, df->loop_start);
    return true;
 }
 
@@ -257,8 +258,10 @@ static ALLEGRO_AUDIO_STREAM *modaudio_stream_init(ALLEGRO_FILE* f,
    int64_t start_pos = -1;
 
    df = lib.dumbfile_open_ex(f, &dfs_f);
-   if (!df)
+   if (!df) {
+      ALLEGRO_ERROR("dumbfile_open_ex failed.\n");
       return NULL;
+   }
 
    start_pos = al_ftell(f);
 
@@ -278,11 +281,13 @@ static ALLEGRO_AUDIO_STREAM *modaudio_stream_init(ALLEGRO_FILE* f,
    duh = loader(df);
 #endif
    if (!duh) {
+      ALLEGRO_ERROR("Failed to create DUH.\n");
       goto Error;
    }
 
    sig = lib.duh_start_sigrenderer(duh, 0, 2, 0);
    if (!sig) {
+      ALLEGRO_ERROR("duh_start_sigrenderer failed.\n");
       goto Error;
    }
 
@@ -317,6 +322,7 @@ static ALLEGRO_AUDIO_STREAM *modaudio_stream_init(ALLEGRO_FILE* f,
       _al_acodec_start_feed_thread(stream);
    }
    else {
+      ALLEGRO_ERROR("Failed to create stream.\n");
       goto Error;
    }
 
@@ -363,7 +369,7 @@ static bool init_libdumb(void)
 #ifdef ALLEGRO_CFG_ACODEC_DUMB_DLL
    dumb_dll = _al_open_library(ALLEGRO_CFG_ACODEC_DUMB_DLL);
    if (!dumb_dll) {
-      ALLEGRO_WARN("Could not load " ALLEGRO_CFG_ACODEC_DUMB_DLL "\n");
+      ALLEGRO_ERROR("Could not load " ALLEGRO_CFG_ACODEC_DUMB_DLL "\n");
       return false;
    }
 
@@ -514,8 +520,10 @@ static ALLEGRO_AUDIO_STREAM *load_mod_audio_stream(const char *filename,
    ASSERT(filename);
 
    f = al_fopen(filename, "rb");
-   if (!f)
+   if (!f) {
+      ALLEGRO_ERROR("Unable to open %s for reading.\n", filename);
       return NULL;
+   }
 
    stream = load_mod_audio_stream_f(f, buffer_count, samples);
 
@@ -537,8 +545,10 @@ static ALLEGRO_AUDIO_STREAM *load_it_audio_stream(const char *filename,
    ASSERT(filename);
 
    f = al_fopen(filename, "rb");
-   if (!f)
+   if (!f) {
+      ALLEGRO_ERROR("Unable to open %s for reading.\n", filename);
       return NULL;
+   }
 
    stream = load_it_audio_stream_f(f, buffer_count, samples);
 
@@ -560,8 +570,10 @@ static ALLEGRO_AUDIO_STREAM *load_xm_audio_stream(const char *filename,
    ASSERT(filename);
 
    f = al_fopen(filename, "rb");
-   if (!f)
+   if (!f) {
+      ALLEGRO_ERROR("Unable to open %s for reading.\n", filename);
       return NULL;
+   }
 
    stream = load_xm_audio_stream_f(f, buffer_count, samples);
 
@@ -583,8 +595,10 @@ static ALLEGRO_AUDIO_STREAM *load_s3m_audio_stream(const char *filename,
    ASSERT(filename);
 
    f = al_fopen(filename, "rb");
-   if (!f)
+   if (!f) {
+      ALLEGRO_ERROR("Unable to open %s for reading.\n", filename);
       return NULL;
+   }
 
    stream = load_s3m_audio_stream_f(f, buffer_count, samples);
 
