@@ -12,6 +12,7 @@
 #include "allegro5/internal/aintern_ttf_cfg.h"
 #include "allegro5/internal/aintern_dtor.h"
 #include "allegro5/internal/aintern_system.h"
+#include "allegro5/internal/aintern_exitfunc.h"
 
 #include <ft2build.h>
 #include FT_FREETYPE_H
@@ -1125,6 +1126,8 @@ bool al_init_ttf_addon(void)
 
    al_register_font_loader(".ttf", al_load_ttf_font);
 
+   _al_add_exit_func(al_shutdown_ttf_addon, "al_shutdown_ttf_addon");
+
    /* Can't fail right now - in the future we might dynamically load
     * the FreeType DLL here and/or initialize FreeType (which both
     * could fail and would cause a false return).
