@@ -275,11 +275,11 @@ final class Key
       ALLEGRO_KEY_BUTTON_R1,   // KeyEvent.KEYCODE_BUTTON_R1
       ALLEGRO_KEY_BUTTON_L2,   // KeyEvent.KEYCODE_BUTTON_L2
       ALLEGRO_KEY_BUTTON_R2,   // KeyEvent.KEYCODE_BUTTON_R2
-      ALLEGRO_KEY_THUMBL,      // KeyEvent.KEYCODE_BUTTON_THUMBL
-      ALLEGRO_KEY_THUMBR,      // KeyEvent.KEYCODE_BUTTON_THUMBR
-      ALLEGRO_KEY_START,       // KeyEvent.KEYCODE_BUTTON_START
-      ALLEGRO_KEY_SELECT,      // KeyEvent.KEYCODE_BUTTON_SELECT
-      ALLEGRO_KEY_UNKNOWN,     // KeyEvent.KEYCODE_BUTTON_MODE
+      ALLEGRO_KEY_THUMBL,      // KeyEvent.KEYCODE_THUMBL
+      ALLEGRO_KEY_THUMBR,      // KeyEvent.KEYCODE_THUMBR
+      ALLEGRO_KEY_START,       // KeyEvent.KEYCODE_START
+      ALLEGRO_KEY_SELECT,      // KeyEvent.KEYCODE_SELECT
+      ALLEGRO_KEY_UNKNOWN,     // KeyEvent.KEYCODE_MODE
       ALLEGRO_KEY_ESCAPE,      // KeyEvent.KEYCODE_ESCAPE (111)
       ALLEGRO_KEY_DELETE,      // KeyEvent.KEYCODE_FORWARD_DELETE (112)
       ALLEGRO_KEY_LCTRL,       // KeyEvent.KEYCODE_CTRL_LEFT (113)
@@ -429,10 +429,7 @@ final class Key
 
    /* Return Allegro key code for Android key code. */
    static int alKey(int keyCode) {
-      if (keyCode < keyMap.length)
-         return keyMap[keyCode];
-      else
-         return ALLEGRO_KEY_UNKNOWN;
+      return keyMap[keyCode];
    }
 }
 
