@@ -5,17 +5,19 @@
  */
 
 #include <allegro5/allegro.h>
+#include <allegro5/allegro_font.h>
 #include <allegro5/allegro_primitives.h>
 
 #include "common.c"
 
-#define MAX_STICKS   8
 #define MAX_AXES     3
+#define MAX_STICKS   16
 #define MAX_BUTTONS  32
 
 
 /* globals */
 ALLEGRO_EVENT_QUEUE  *event_queue;
+ALLEGRO_FONT         *font;
 ALLEGRO_COLOR        black;
 ALLEGRO_COLOR        grey;
 ALLEGRO_COLOR        white;
@@ -59,7 +61,8 @@ static void setup_joystick_values(ALLEGRO_JOYSTICK *joy)
 }
 
 
-static void draw_joystick_axes(int cx, int cy, int stick)
+
+static void draw_joystick_axes(ALLEGRO_JOYSTICK *joy, int cx, int cy, int stick)
 {
    const int size = 30;
    const int csize = 5;
@@ -68,36 +71,59 @@ static void draw_joystick_axes(int cx, int cy, int stick)
    int x = cx + joys[stick][0] * size;
    int y = cy + joys[stick][1] * size;
    int z = cy + joys[stick][2] * size;
+   int i;
 
    al_draw_filled_rectangle(cx-osize, cy-osize, cx+osize, cy+osize, grey);
    al_draw_rectangle(cx-osize+0.5, cy-osize+0.5, cx+osize-0.5, cy+osize-0.5, black, 0);
    al_draw_filled_rectangle(x-5, y-5, x+5, y+5, black);
 
-   if (num_axes[stick] == 3) {
+   if (num_axes[stick] >= 3) {
       al_draw_filled_rectangle(zx-csize, cy-osize, zx+csize, cy+osize, grey);
       al_draw_rectangle(zx-csize+0.5f, cy-osize+0.5f, zx+csize-0.5f, cy+osize-0.5f, black, 0);
       al_draw_filled_rectangle(zx-5, z-5, zx+5, z+5, black);
    }
+
+   if (joy) {
+      al_draw_text(font, black, cx, cy + osize + 1, ALLEGRO_ALIGN_CENTRE,
+         al_get_joystick_stick_name(joy, stick));
+      for (i = 0; i < num_axes[stick]; i++) {
+         al_draw_text(font, black, cx, cy + osize + (1 + i) * 10,
+            ALLEGRO_ALIGN_CENTRE,
+            al_get_joystick_axis_name(joy, stick, i));
+      }
+   }
 }
 
 
 
-static void draw_joystick_button(int button, bool down)
+static void draw_joystick_button(ALLEGRO_JOYSTICK *joy, int button, bool down)
 {
    ALLEGRO_BITMAP *bmp = al_get_target_bitmap();
    int x = al_get_bitmap_width(bmp)/2-120 + (button % 8) * 30;
    int y = al_get_bitmap_height(bmp)-120 + (button / 8) * 30;
+   ALLEGRO_COLOR fg;
 
    al_draw_filled_rectangle(x, y, x + 25, y + 25, grey);
    al_draw_rectangle(x+0.5, y+0.5, x + 24.5, y + 24.5, black, 0);
    if (down) {
       al_draw_filled_rectangle(x + 2, y + 2, x + 23, y + 23, black);
+      fg = white;
+   }
+   else {
+      fg = black;
+   }
+
+   if (joy) {
+      const char *name = al_get_joystick_button_name(joy, button);
+      if (strlen(name) < 4) {
+         al_draw_text(font, fg, x + 13, y + 8, ALLEGRO_ALIGN_CENTRE, name);
+      }
    }
 }
 
 
 
-static void draw_all(void)
+static void draw_all(ALLEGRO_JOYSTICK *joy)
 {
    ALLEGRO_BITMAP *bmp = al_get_target_bitmap();
    int width = al_get_bitmap_width(bmp);
@@ -106,14 +132,21 @@ static void draw_all(void)
 
    al_clear_to_color(al_map_rgb(0xff, 0xff, 0xc0));
 
+   if (joy) {
+      al_draw_textf(font, black, width / 2, 10, ALLEGRO_ALIGN_CENTRE,
+         "Joystick: %s", al_get_joystick_name(joy));
+   }
+
    for (i = 0; i < num_sticks; i++) {
-      int cx = (i + 0.5) * width / num_sticks;
-      int cy = height / 2;
-      draw_joystick_axes(cx, cy, i);
+      int u = i%4;
+      int v = i/4;
+      int cx = (u + 0.5) * width/4;
+      int cy = (v + 0.5) * height/6;
+      draw_joystick_axes(joy, cx, cy, i);
    }
 
    for (i = 0; i < num_buttons; i++) {
-      draw_joystick_button(i, joys_buttons[i]);
+      draw_joystick_button(joy, i, joys_buttons[i]);
    }
 
    al_flip_display();
@@ -127,7 +160,7 @@ static void main_loop(void)
 
    while (true) {
       if (al_is_event_queue_empty(event_queue))
-         draw_all();
+         draw_all(al_get_joystick(0));
 
       al_wait_for_event(event_queue, &event);
 
@@ -178,16 +211,20 @@ static void main_loop(void)
 
 
 
-int main(void)
+int main(int argc, char **argv)
 {
    ALLEGRO_DISPLAY *display;
 
+   (void)argc;
+   (void)argv;
+
    if (!al_init()) {
       abort_example("Could not init Allegro.\n");
    }
    al_init_primitives_addon();
+   al_init_font_addon();
 
-   display = al_create_display(640, 480);
+   display = al_create_display(1024, 768);
    if (!display) {
       abort_example("al_create_display failed\n");
    }
@@ -197,6 +234,7 @@ int main(void)
    black = al_map_rgb(0, 0, 0);
    grey = al_map_rgb(0xe0, 0xe0, 0xe0);
    white = al_map_rgb(255, 255, 255);
+   font = al_create_builtin_font();
 
    al_install_joystick();
 
@@ -215,6 +253,8 @@ int main(void)
 
    main_loop();
 
+   al_destroy_font(font);
+
    return 0;
 }
 
