@@ -193,13 +193,11 @@ ALLEGRO_BITMAP *al_load_bitmap_flags(const char *filename, int flags)
    Handler *h;
    ALLEGRO_BITMAP *ret;
 
-   ext = strrchr(filename, '.');
+   ext = al_identify_bitmap(filename);
    if (!ext) {
-      ext = al_identify_bitmap(filename);
+      ext = strrchr(filename, '.');
       if (!ext) {
-         ALLEGRO_WARN("Bitmap %s has no extension and filetype "
-            "identification failed - not even trying to load it.\n",
-            filename);
+         ALLEGRO_ERROR("Could not identify bitmap %s!", filename);
          return NULL;
       }
    }
@@ -208,12 +206,11 @@ ALLEGRO_BITMAP *al_load_bitmap_flags(const char *filename, int flags)
    if (h && h->loader) {
       ret = h->loader(filename, flags);
       if (!ret)
-         ALLEGRO_WARN("Failed loading %s with %s handler.\n", filename,
-            ext);
+         ALLEGRO_ERROR("Failed loading bitmap %s with %s handler.\n",
+            filename, ext);
    }
    else {
-      ALLEGRO_WARN("No handler for bitmap extension %s - "
-         "therefore not trying to load %s.\n", ext, filename);
+      ALLEGRO_ERROR("No handler for bitmap %s!", filename);
       ret = NULL;
    }
 
@@ -229,14 +226,16 @@ bool al_save_bitmap(const char *filename, ALLEGRO_BITMAP *bitmap)
    Handler *h;
 
    ext = strrchr(filename, '.');
-   if (!ext)
+   if (!ext) {
+      ALLEGRO_ERROR("Unable to determine file format from %s\n", filename);
       return false;
+   }
 
    h = find_handler(ext, false);
    if (h && h->saver)
       return h->saver(filename, bitmap);
    else {
-      ALLEGRO_WARN("No handler for image %s found\n", filename);
+      ALLEGRO_ERROR("No handler for image %s found\n", filename);
       return false;
    }
 }
@@ -285,7 +284,7 @@ bool al_save_bitmap_f(ALLEGRO_FILE *fp, const char *ident,
    if (h && h->fs_saver)
       return h->fs_saver(fp, bitmap);
    else {
-      ALLEGRO_WARN("No handler for image %s found\n", ident);
+      ALLEGRO_ERROR("No handler for image %s found\n", ident);
       return false;
    }
 }
