@@ -16,6 +16,10 @@
  */
 
 
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
+
 #include <math.h>
 #include <stdio.h>
 #include <sys/time.h>
