@@ -66,7 +66,9 @@ static void osx_tell_dock(void)
 {
    ProcessSerialNumber psn = { 0, kCurrentProcess };
    TransformProcessType(&psn, kProcessTransformToForegroundApplication);
-   [[NSApplication sharedApplication] activateIgnoringOtherApps: YES];
+   [[NSApplication sharedApplication] performSelectorOnMainThread: @selector(activateIgnoringOtherApps:)
+      withObject: [NSNumber numberWithBool:YES]
+      waitUntilDone: YES];
 }
 
 
@@ -365,17 +367,23 @@ static int osx_get_num_video_adapters(void)
  */
 static bool osx_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO* info)
 {
-/*
    int count = osx_get_num_video_adapters();
    if (adapter < count) {
       NSScreen *screen = [[NSScreen screens] objectAtIndex: adapter];
       NSRect rc = [screen frame];
+      rc = [screen convertRectToBacking: rc];
       info->x1 = (int) rc.origin.x;
       info->x2 = (int) (rc.origin.x + rc.size.width);
       info->y1 = (int) rc.origin.y;
       info->y2 = (int) (rc.origin.y + rc.size.height);
+      ALLEGRO_INFO("Display %d has coordinates (%d, %d) - (%d, %d)\n",
+                   adapter, info->x1, info->y1, info->x2, info->y2);
+      return true;
    }
-*/
+   else {
+      return false;
+   }
+/*
    CGDisplayCount count;
    // Assume no more than 16 monitors connected
    static const int max_displays = 16;
@@ -394,6 +402,30 @@ static bool osx_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO* info)
    else {
       return false;
    }
+*/
+}
+
+/* osx_get_monitor_dpi:
+ * Return the dots per inch value of one monitor
+ */
+static int osx_get_monitor_dpi(int adapter)
+{
+   int count = osx_get_num_video_adapters();
+   if (adapter < count) {
+      NSScreen *screen = [[NSScreen screens] objectAtIndex: adapter];
+      NSRect rc = [screen frame];
+      rc = [screen convertRectToBacking: rc];
+
+      NSDictionary *description = [screen deviceDescription];
+      CGSize size = CGDisplayScreenSize([[description objectForKey:@"NSScreenNumber"] unsignedIntValue]);
+      float dpi_hori = rc.size.width / (_AL_INCHES_PER_MM * size.width);
+      float dpi_vert = rc.size.height / (_AL_INCHES_PER_MM * size.height);
+
+      return sqrt(dpi_hori * dpi_vert);
+   }
+   else {
+      return 0;
+   }
 }
 
 /* osx_inhibit_screensaver:
@@ -405,9 +437,10 @@ static bool osx_get_monitor_info(int adapter, ALLEGRO_MONITOR_INFO* info)
 static bool osx_inhibit_screensaver(bool inhibit)
 {
    // Send a message to the App's delegate always on the main thread
-   [[NSApp delegate] performSelectorOnMainThread: @selector(setInhibitScreenSaver:)
-      withObject: [NSNumber numberWithBool:inhibit ? YES : NO]
-      waitUntilDone: NO];
+   NSObject* delegate = [NSApp delegate];
+   [delegate performSelectorOnMainThread: @selector(setInhibitScreenSaver:)
+                              withObject: [NSNumber numberWithBool:inhibit ? YES : NO]
+                           waitUntilDone: NO];
    ALLEGRO_INFO("Stop screensaver\n");
    return true;
 }
@@ -492,6 +525,7 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_osx_driver(void)
    if (vt == NULL) {
       vt = al_malloc(sizeof(*vt));
       memset(vt, 0, sizeof(*vt));
+      vt->id = ALLEGRO_SYSTEM_ID_MACOSX;
       vt->initialize = osx_sys_init;
       vt->get_display_driver = _al_osx_get_display_driver;
       vt->get_keyboard_driver = _al_osx_get_keyboard_driver;
@@ -502,6 +536,7 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_osx_driver(void)
       vt->shutdown_system = osx_sys_exit;
       vt->get_num_video_adapters = osx_get_num_video_adapters;
       vt->get_monitor_info = osx_get_monitor_info;
+      vt->get_monitor_dpi = osx_get_monitor_dpi;
       vt->create_mouse_cursor = _al_osx_create_mouse_cursor;
       vt->destroy_mouse_cursor = _al_osx_destroy_mouse_cursor;
       vt->get_cursor_position = osx_get_cursor_position;
@@ -509,6 +544,9 @@ ALLEGRO_SYSTEM_INTERFACE *_al_system_osx_driver(void)
       vt->inhibit_screensaver = osx_inhibit_screensaver;
       vt->thread_init = osx_thread_init;
       vt->thread_exit = osx_thread_exit;
+      vt->get_time = _al_unix_get_time;
+      vt->rest = _al_unix_rest;
+      vt->init_timeout = _al_unix_init_timeout;
 
    };
 
