@@ -14,8 +14,6 @@
  *
  */
 
-#define UNICODE
-
 #include <windows.h>
 
 #include <string.h>
@@ -39,6 +37,21 @@
 
 #include "d3d.h"
 
+// Based on the code in <VersionHelpers.h> but that header is not
+// available on Windows 7 SDK and earlier.
+static BOOL is_windows_vista_or_greater() {
+   OSVERSIONINFOEX info;
+   ULONGLONG mask = 0;
+   VER_SET_CONDITION(mask, VER_MAJORVERSION, VER_GREATER_EQUAL);
+   VER_SET_CONDITION(mask, VER_MINORVERSION, VER_GREATER_EQUAL);
+   VER_SET_CONDITION(mask, VER_SERVICEPACKMAJOR, VER_GREATER_EQUAL);
+   memset(&info, 0, sizeof(info));
+   info.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
+   info.dwMajorVersion = 6;
+
+   return VerifyVersionInfo(&info, VER_MAJORVERSION | VER_MINORVERSION | VER_SERVICEPACKMAJOR, mask) != FALSE;
+}
+
 static void d3d_set_target_bitmap(ALLEGRO_DISPLAY *display, ALLEGRO_BITMAP *bitmap);
 static void d3d_update_transformation(ALLEGRO_DISPLAY* disp, ALLEGRO_BITMAP *target);
 static bool d3d_init_display();
@@ -608,7 +621,6 @@ bool _al_d3d_render_to_texture_supported(void)
 static bool d3d_init_display()
 {
    D3DDISPLAYMODE d3d_dm;
-   OSVERSIONINFO info;
 
    _al_d3d_module = _al_win_safe_load_library(_al_d3d_module_name);
    if (_al_d3d_module == NULL) {
@@ -616,9 +628,7 @@ static bool d3d_init_display()
       return false;
    }
 
-   info.dwOSVersionInfoSize = sizeof(info);
-   GetVersionEx(&info);
-   is_vista = info.dwMajorVersion >= 6;
+   is_vista = is_windows_vista_or_greater();
 
 #ifdef ALLEGRO_CFG_D3D9EX
    if (is_vista) {
@@ -796,6 +806,10 @@ static bool d3d_create_device(ALLEGRO_DISPLAY_D3D *d,
    }
    d3d_pp.hDeviceWindow = win_display->window;
 
+   if (!refresh_rate) {
+      al_display->refresh_rate = d3d_get_default_refresh_rate(win_display->adapter);
+   }
+
    if (adapter < 0)
       adapter = 0;
 
@@ -1847,7 +1861,7 @@ static ALLEGRO_DISPLAY *d3d_create_display(int w, int h)
    s[ALLEGRO_SUPPORT_NPOT_BITMAP] = al_have_d3d_non_pow2_texture_support();
    s[ALLEGRO_CAN_DRAW_INTO_BITMAP] = render_to_texture_supported;
 
-#ifdef ALLEGRO_CFG_SHADER_HLSL
+#ifdef ALLEGRO_CFG_D3DX9
    _al_load_d3dx9_module();
 #endif
 
@@ -2277,7 +2291,7 @@ static bool d3d_resize_helper(ALLEGRO_DISPLAY *d, int width, int height)
       wi.cbSize = sizeof(WINDOWINFO);
       GetWindowInfo(win_display->window, &wi);
 
-      AdjustWindowRectEx(&win_size, wi.dwStyle, false, wi.dwExStyle);
+      AdjustWindowRectEx(&win_size, wi.dwStyle, GetMenu(win_display->window) ? TRUE : FALSE, wi.dwExStyle);
 
       // FIXME: Handle failure (for example if window constraints are active?)
       SetWindowPos(win_display->window, HWND_TOP,
@@ -2640,7 +2654,7 @@ void _al_d3d_shutdown_display(void)
    FreeLibrary(_al_d3d_module);
    _al_d3d_module = NULL;
 
-#ifdef ALLEGRO_CFG_SHADER_HLSL
+#ifdef ALLEGRO_CFG_D3DX9
    _al_unload_d3dx9_module();
 #endif
 
